#include <iostream>
#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include "artdaq/Application/TaskType.hh"
#include "artdaq/DAQdata/Globals.hh"
#include "artdaq/Application/DataLoggerApp.hh"
#include "artdaq/ExternalComms/MakeCommanderPlugin.hh"
#include "artdaq/Application/LoadParameterSet.hh"
#include "artdaq-mpich-plugin/Application/MPISentry.hh"
#include "artdaq/BuildInfo/GetPackageBuildInfo.hh"
#include "cetlib_except/exception.h"

int main(int argc, char* argv[])
{
	// initialization

	int const wanted_threading_level{ MPI_THREAD_FUNNELED };

	MPI_Comm local_group_comm;
	std::unique_ptr<artdaq::MPISentry> mpiSentry;

	try
	{
		mpiSentry.reset(new artdaq::MPISentry(&argc, &argv, wanted_threading_level, artdaq::TaskType::DataLoggerTask, local_group_comm));
	}
	catch (cet::exception& errormsg)
	{
		TLOG_ERROR("DataLoggerMain") << errormsg ;
		TLOG_ERROR("DataLoggerMain") << "MPISentry error encountered in DataLoggerMain; exiting..." ;
		throw errormsg;
	}

	struct Config
	{
		fhicl::TableFragment<artdaq::CommanderInterface::Config> commander_config;
		fhicl::Atom<std::string> application_name{ fhicl::Name{ "application_name" }, fhicl::Comment{ "Name of the application for metrics and logging" }, "DataLogger" };
	};

	fhicl::ParameterSet config = LoadParameterSet<Config>(argc, argv, "DataLoggerMain", "This is the DataLogger's main executable (MPI version)");
	app_name = config.get<std::string>("application_name", "DataLogger");
	std::string mf_app_name = artdaq::setMsgFacAppName(app_name, config.get<int>("id"));
	artdaq::configureMessageFacility(mf_app_name.c_str());
	TLOG_DEBUG(app_name + "Main") << "Setting application name to " << mf_app_name ;

	// 23-May-2018, KAB: added lookup of the partition number from the command line arguments.
	if (config.has_key("partition_number"))
	{
		artdaq::Globals::partition_number_ = config.get<int>("partition_number");
	}
	TLOG_DEBUG(app_name + "Main") << "Setting partition number to " << artdaq::Globals::partition_number_;

	TLOG_INFO(app_name + "Main") << "artdaq version " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getPackageVersion()
		<< ", built " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getBuildTimestamp() ;

	// create the DataLoggerApp
	my_rank = mpiSentry->rank();
	artdaq::DataLoggerApp dl_app;

	auto commander = artdaq::MakeCommanderPlugin(config, dl_app);
	TLOG_INFO(app_name + "Main") << "Running Commmander Server" ;
	commander->run_server();
	TLOG_INFO(app_name + "Main") << "Commandable Server ended, exiting..." ;
}
